@@ -2,7 +2,6 @@
 #include <vector>
 #include <RcppEigen.h>
 #include <Rcpp.h>
-#include <RcppEigen.h>
 using namespace Rcpp;
 using namespace std;
 using namespace Eigen;
@@ -16,25 +15,33 @@ double thresholdl1(double x, double thr);
 List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int nlambda, int d)
 {
 
-    Eigen::ArrayXd Xb, r, grad, w1, Y, XX, gr;
+    Eigen::ArrayXd Xb, r, grad, w1, Y, gr;
     Eigen::ArrayXXd X;
     Eigen::MatrixXd tmp_icov;
     tmp_icov.resize(d, d);
     tmp_icov.setZero();
     std::vector<Eigen::MatrixXd > tmp_icov_p;
     tmp_icov_p.clear();
+    tmp_icov_p.reserve(nlambda > 0 ? nlambda : 0);
     for(int i = 0; i < nlambda; i++)
       tmp_icov_p.push_back(tmp_icov);
     int n = data.rows();
+    if(n <= 0 || d <= 0 || nlambda <= 0)
+    {
+      int d_safe = d > 0 ? d : 0;
+      return List::create(
+        _["col_cnz"] = IntegerVector(d_safe + 1),
+        _["row_idx"] = IntegerVector(0),
+        _["x"] = NumericVector(0),
+        _["icov"] = std::vector<Eigen::MatrixXd>()
+      );
+    }
     int maxdf = 0;
-    maxdf = (n < d ? n : d)*d;
+    maxdf = (n < d ? n : d);
     NumericVector x(d*maxdf*nlambda);
     IntegerVector col_cnz(d+1);
     IntegerVector row_idx(d*maxdf*nlambda);
     X = data;
-    XX.resize(d);
-    for (int j = 0; j < d; j++)
-      XX[j] = (X.col(j)*X.col(j)).sum()/n;
     double prec = 1e-4;
     int max_iter = 1000;
     int num_relaxation_round = 3;
@@ -54,20 +61,27 @@ List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int n
       Y = X.col(m);
 
       Eigen::ArrayXd Xb_master(n);
-      Eigen::ArrayXd w1_master(n);
+      Eigen::ArrayXd w1_master(d);
       std::vector<int> actset_indcat(d, 0);
       std::vector<int> actset_indcat_master(d, 0);
       std::vector<int> actset_idx;
       std::vector<double> old_coef(d, 0);
-      std::vector<double> grad(d, 0);
       std::vector<double> grad_master(d, 0);
 
-      double a = 0, g = 0, L = 0, sum_r2 = 0;
+      double L = 0, sum_r2 = 0;
       double tmp_change = 0, local_change = 0;
+      const double eps = 1e-12;
+      auto refresh_residual = [&]() {
+        sum_r2 = r.matrix().dot(r.matrix());
+        if (sum_r2 < eps)
+          sum_r2 = eps;
+        L = sqrt(sum_r2 / n);
+        if (L < eps)
+          L = eps;
+      };
 
       r = Y - Xb;
-      sum_r2 = r.matrix().dot(r.matrix());
-      L = sqrt(sum_r2 / n);
+      refresh_residual();
 
       double dev_thr = fabs(L) * prec;
 
@@ -78,15 +92,14 @@ List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int n
       {
         grad[i] = (r * X.col(i)).sum() / (n*L);
       }
-      for(int i = 0; i < d; i++) gr[i] = abs(grad[i]);
+      for(int i = 0; i < d; i++) gr[i] = fabs(grad[i]);
       w1_master = w1;
       Xb_master = Xb;
       for (int i = 0; i < d; i++) grad_master[i] = gr[i];
 
-      std::vector<double> stage_lambdas(d, 0);
-
       for(int i=0;i<nlambda;i++)
       {
+        const double stage_lambda = lambda[i];
         w1 = w1_master;
         Xb = Xb_master;
 
@@ -103,26 +116,15 @@ List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int n
         else
           threshold = 2 * lambda[i];
 
-        for (int j = 0; j < m; ++j)
-        {
-          stage_lambdas[j] = lambda[i];
-
-          if (gr[j] > threshold) actset_indcat[j] = 1;
-        }
-        for (int j = m+1; j < d; ++j)
-        {
-          stage_lambdas[j] = lambda[i];
-
-          if (gr[j] > threshold) actset_indcat[j] = 1;
-        }
-        stage_lambdas[m] = lambda[i];
+        for (int j = 0; j < d; ++j)
+          if (j != m && gr[j] > threshold)
+            actset_indcat[j] = 1;
         r = Y - Xb;
-        sum_r2 = r.matrix().dot(r.matrix());
-        L = sqrt(sum_r2 / n);
+        refresh_residual();
         // loop level 0: multistage convex relaxation
         int loopcnt_level_0 = 0;
         int idx;
-        double old_w1, updated_coord;
+        double old_w1;
         while (loopcnt_level_0 < num_relaxation_round)
         {
           loopcnt_level_0++;
@@ -130,82 +132,42 @@ List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int n
           // loop level 1: active set update
           int loopcnt_level_1 = 0;
           bool terminate_loop_level_1 = true;
+          Eigen::ArrayXd wXX(n);
           while (loopcnt_level_1 < max_iter)
           {
             loopcnt_level_1++;
             terminate_loop_level_1 = true;
 
             for (int j = 0; j < d; j++) old_coef[j] = w1[j];
+            refresh_residual();
 
             // initialize actset_idx
             actset_idx.clear();
-            for (int j = 0; j < m; j++)
-              if (actset_indcat[j])
-              {
-                g = 0.0;
-                a = 0.0;
+            auto update_coordinate = [&](int coord_idx) {
+              wXX = (1 - r*r/sum_r2) * X.col(coord_idx) * X.col(coord_idx);
+              double g = (wXX * w1[coord_idx] + r * X.col(coord_idx)).sum()/(n*L);
+              double a = wXX.sum()/(n*L);
+              double tmp = w1[coord_idx];
+              if (fabs(a) > eps)
+                w1[coord_idx] = thresholdl1(g, stage_lambda) / a;
+              else
+                w1[coord_idx] = 0;
 
-                double tmp;
-
-                sum_r2 = r.matrix().dot(r.matrix());
-                L = sqrt(sum_r2 / n);
-
-                Eigen::ArrayXd wXX  = (1 - r*r/sum_r2) * X.col(j) * X.col(j);
-                g = (wXX * w1[j] + r * X.col(j)).sum()/(n*L);
-                a = wXX.sum()/(n*L);
-
-                tmp = w1[j];
-                w1[j] = thresholdl1(g, stage_lambdas[j]) / a;
-
-                tmp = w1[j] - tmp;
-                // Xb += delta*X[idx*n]
-                Xb = Xb + tmp * X.col(j);
-
-                sum_r2 = 0.0;
-                // r -= delta*X
-                r = r - tmp * X.col(j);
-
-                sum_r2 = r.matrix().dot(r.matrix());
-                L = sqrt(sum_r2 / n);
-
-                updated_coord = w1[j];
-
-                if (fabs(updated_coord) > 0) actset_idx.push_back(j);
+              tmp = w1[coord_idx] - tmp;
+              if (tmp != 0) {
+                Xb = Xb + tmp * X.col(coord_idx);
+                r = r - tmp * X.col(coord_idx);
+                refresh_residual();
               }
+            };
 
-            for (int j = m+1; j < d; j++)
-              if (actset_indcat[j])
-              {
-                  g = 0.0;
-                  a = 0.0;
-
-                  double tmp;
-
-                  sum_r2 = r.matrix().dot(r.matrix());
-                  L = sqrt(sum_r2 / n);
-
-                  Eigen::ArrayXd wXX  = (1 - r*r/sum_r2) * X.col(j) * X.col(j);
-                  g = (wXX * w1[j] + r * X.col(j)).sum()/(n*L);
-                  a = wXX.sum()/(n*L);
-
-                  tmp = w1[j];
-                  w1[j] = thresholdl1(g, stage_lambdas[j]) / a;
-
-                  tmp = w1[j] - tmp;
-                  // Xb += delta*X[idx*n]
-                  Xb = Xb + tmp * X.col(j);
-
-                  sum_r2 = 0.0;
-                  // r -= delta*X
-                  r = r - tmp * X.col(j);
-
-                  sum_r2 = r.matrix().dot(r.matrix());
-                  L = sqrt(sum_r2 / n);
-
-                  updated_coord = w1[j];
-
-                  if (fabs(updated_coord) > 0) actset_idx.push_back(j);
-                }
+            for (int j = 0; j < d; ++j) {
+              if (j == m || !actset_indcat[j])
+                continue;
+              update_coordinate(j);
+              if (fabs(w1[j]) > 0)
+                actset_idx.push_back(j);
+            }
 
               // loop level 2: proximal newton on active set
             int loopcnt_level_2 = 0;
@@ -220,36 +182,10 @@ List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int n
                   idx = actset_idx[k];
 
                   old_w1 = w1[idx];
-                  g = 0.0;
-                  a = 0.0;
-
-                  double tmp;
-
-                  sum_r2 = r.matrix().dot(r.matrix());
-                  L = sqrt(sum_r2 / n);
-
-                  Eigen::ArrayXd wXX  = (1 - r*r/sum_r2) * X.col(idx) * X.col(idx);
-                  g = (wXX * w1[idx] + r * X.col(idx)).sum()/(n*L);
-                  a = wXX.sum()/(n*L);
-
-                  tmp = w1[idx];
-                  w1[idx] = thresholdl1(g, stage_lambdas[idx]) / a;
-
-                  tmp = w1[idx] - tmp;
-                  // Xb += delta*X[idx*n]
-                  Xb = Xb + tmp * X.col(idx);
-
-                  sum_r2 = 0.0;
-                  // r -= delta*X
-                  r = r - tmp * X.col(idx);
-
-                  sum_r2 = r.matrix().dot(r.matrix());
-                  L = sqrt(sum_r2 / n);
-
-                  updated_coord = w1[idx];
+                  update_coordinate(idx);
                   tmp_change = old_w1 - w1[idx];
-                  double a =  (X.col(idx) * X.col(idx) * (1 - r * r/(L*L*n))).sum()/(n*L);
-                  local_change = a * tmp_change * tmp_change / (2 * L * n);
+                  double h = fabs((X.col(idx) * X.col(idx) * (1 - r * r/(L*L*n))).sum()/(n*L));
+                  local_change = h * tmp_change * tmp_change / (2 * L * n);
                   if (local_change > dev_thr)
                     terminate_loop_level_2 = false;
                 }
@@ -262,16 +198,15 @@ List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int n
             for (unsigned int k = 0; k < actset_idx.size(); ++k)
             {
               idx = actset_idx[k];
-              tmp_change = old_w1 - w1[idx];
-              double a =  (X.col(idx) * X.col(idx) * (1 - r * r/(L*L*n))).sum()/(n*L);
-              local_change = a * tmp_change * tmp_change / (2 * L * n);
+              tmp_change = old_coef[idx] - w1[idx];
+              double h = fabs((X.col(idx) * X.col(idx) * (1 - r * r/(L*L*n))).sum()/(n*L));
+              local_change = h * tmp_change * tmp_change / (2 * L * n);
               if (local_change > dev_thr)
                 terminate_loop_level_1 = false;
             }
 
             r = Y - Xb;
-            sum_r2 = r.matrix().dot(r.matrix());
-            L = sqrt(sum_r2 / n);
+            refresh_residual();
 
             if (terminate_loop_level_1)
               break;
@@ -279,25 +214,12 @@ List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int n
 
               // check stopping criterion 2: active set change
             bool new_active_idx = false;
-            for (int k = 0; k < m; k++)
-              if (actset_indcat[k] == 0)
+            for (int k = 0; k < d; ++k)
+              if (k != m && actset_indcat[k] == 0)
               {
-                grad[idx] = (r * X.col(idx)).sum() / (n*L);
-                //cout<<grad[idx];
+                grad[k] = (r * X.col(k)).sum() / (n*L);
                 gr[k] = fabs(grad[k]);
-                if (gr[k] > stage_lambdas[k])
-                {
-                  actset_indcat[k] = 1;
-                  new_active_idx = true;
-                }
-              }
-            for (int k = m+1; k < d; k++)
-              if (actset_indcat[k] == 0)
-              {
-                grad[idx] = (r * X.col(idx)).sum() / (n*L);
-                //cout<<grad[idx]
-                gr[k] = fabs(grad[k]);
-                if (gr[k] > stage_lambdas[k])
+                if (gr[k] > stage_lambda)
                 {
                   actset_indcat[k] = 1;
                   new_active_idx = true;
@@ -327,28 +249,18 @@ List SPMBgraphsqrt(Eigen::Map<Eigen::MatrixXd> data, NumericVector lambda, int n
           cnz++;
           //cout<<cnz<<"    ";
         }
-        double tal = 0;
-        Eigen::MatrixXd temp;
-        temp.resize(n, 1);
-        for(int j = 0; j < n; j++)
-        {
-          temp(j, 0) = 0;
-          for(int k = 0; k < d; k++)
-            temp(j, 0) += X.matrix()(j, k)*w1[k];
-          temp(j, 0) = Y[j] - temp(j, 0);
-        }
-        //temp = Y.matrix() - X.matrix().transpose()*w1.matrix();
-        for(int j = 0; j < n; j++)
-          tal += temp(j, 0)*temp(j, 0);
-        tal = sqrt(tal)/sqrt(n);
+        r = Y - Xb;
+        refresh_residual();
+        double tal = L;
 
-        tmp_icov = tmp_icov_p[i];
-        tmp_icov(m, m) = pow(tal, -2);
-        for(int j = 0; j < m; j++)
-          tmp_icov(j, m) = -tmp_icov(m, m)*w1[j];
-        for(int j = m+1; j < d; j++)
-          tmp_icov(j, m) = -tmp_icov(m, m)*w1[j];
-        tmp_icov_p[i] = tmp_icov;
+        Eigen::MatrixXd &tmp_icov_ref = tmp_icov_p[i];
+        if(tal > 0)
+          tmp_icov_ref(m, m) = 1.0/(tal*tal);
+        else
+          tmp_icov_ref(m, m) = 0;
+        for(int j = 0; j < d; ++j)
+          if (j != m)
+            tmp_icov_ref(j, m) = -tmp_icov_ref(m, m)*w1[j];
       }
       col_cnz[m+1]=cnz;
     }
